import traits.api as tr
import numpy as np

from bmcs_utils.api import Model, View, Item, Button, ButtonEditor, Float, Int, \
    mpl_align_yaxis, ParametricStudy
from bmcs_utils.mpl_utils import mpl_align_xaxis

from bmcs_beam.beam_config.beam_design import BeamDesign
from bmcs_cross_section.mkappa import MKappa
from scipy.integrate import cumtrapz
import matplotlib.gridspec as gridspec

from bmcs_beam.beam_config.boundary_conditions import BoundaryConditions, BoundaryConfig


class DeflectionProfile(Model):
    '''
    Deflection model of a BMCS beam
    '''

    name = 'Deflection Profile'

    beam_design = tr.Instance(BeamDesign, ())
    mc = tr.Instance(MKappa, ())
    n_load_steps = Int(31)

    tree = ['beam_design','mc']

    ipw_view = View(
        Item('n_load_steps', latex='n_{\mathrm{load~steps}}')
    )

    def get_kappa_x(self):
        '''
        Profile of curvature along the beam
        '''
        M = self.beam_design.get_M_x()
        return self.mc.get_kappa_M(M)
    
    def get_kappa_shrinkage(self):
        '''
        Calculate the shrinkage curvature based on EC2
        '''
        f_ck = 30
        t = 365
        t_s = 3
        
        alpha_ds1 = 4
        alpha_ds2 = 0.12
        
        RH = 0.70
        RH_0 = 1.00
        
        f_cm = 30
        f_cmo = 10
        phi = 2.5
        E_s = 200000
        E_cm = 33000
        S = 1000 * 300 ** 2 / 2
        I = 1000 * 300 ** 3 / 12
        A_c = 1000 * 300
        u = 2 * (1000 + 300)

        eps_ca_infty = 2.5 * (f_ck - 10) * 1e-6    
        beta_as_t = 1 - np.exp(- 0.2 * t ** 0.5)
        eps_ca = beta_as_t * eps_ca_infty

        beta_RH = 1.55 * (1 - (RH/RH_0)**3)
        h_0 = 2 * A_c / u
        beta_ds_t_t_s = (t - t_s) / ((t - t_s) + 0.04 * h_0 ** (3/2))
        h_0_ = [100, 200 ,300, 500, 800]
        k_h_ = [ 1, 0.85, 0.75, 0.7, 0.7 ]
        k_h = np.interp(h_0, h_0_, k_h_)
        eps_cd0 = 0.85 * ((220 + 110 * alpha_ds1) * np.exp(-alpha_ds2 * f_cm / f_cmo)) * 1e-6 * beta_RH    
        eps_cd = beta_ds_t_t_s * k_h * eps_cd0   

        eps_cs = eps_cd + eps_ca

        E_c_eff = E_cm / (1 + phi)
        alpha_e = E_s / E_c_eff
        kappa_cs = eps_cs * alpha_e * S / I

        kappa_cs_ = np.array([kappa_cs])
        kappa_cs_x = np.zeros_like(self.get_kappa_x())
        kappa_cs_x[:] = kappa_cs_
        
        return kappa_cs_x

    def get_phi_x(self):
        '''
        Calculate the cross sectional rotation by integrating the curvature
        '''
        # TODO rename phi to theta
        kappa_x = self.get_kappa_x() # + 2e-6 #+ self.get_kappa_shrinkage()
        # Kappa = 1/R = d_phi/d_x
        phi_x = cumtrapz(kappa_x, self.beam_design.x, initial=0)

        # resolve the integration constant by requiring zero curvature
        # at the midspan of the beam
        # TODO [SD] this is specific to 3 point bending - generalize
        #           for other loading conditions.
        #           HS: I guess this works for 4pb too (for symmetric beams)

        if self.beam_design.beam_conf_name != BoundaryConfig.FIXED_AND_ROLLER_SUPPORT_DIST_LOAD \
                and self.beam_design.beam_conf_name != BoundaryConfig.CANTILEVER_DIST_LOAD:
            phi_x -= np.interp(self.beam_design.L / 2, self.beam_design.x, phi_x)
        elif self.beam_design.beam_conf_name == BoundaryConfig.FIXED_AND_ROLLER_SUPPORT_DIST_LOAD:
            phi_x -= phi_x[-1]

        # This is a more general approach but not sure from it!
        # kappa_derivative = np.gradient(kappa_x, self.beam_design.x)
        # kappa_derivative_signs = np.sign(kappa_derivative)
        # sign_change = (np.diff(kappa_derivative_signs) != 0) * 1
        # if sign_change.size != 1:
        #     print("Warning: multiple curvature slope changes has been found but only the first is considered!!")
        # sign_change_index = np.where(sign_change == 1)[0][0]
        # phi_x -= phi_x[sign_change_index]

        return phi_x

    def get_w_x(self):
        """
        Profile of deflection along the beam
        """
        phi_x = self.get_phi_x()
        w_x = cumtrapz(phi_x, self.beam_design.x, initial=0)
        # resolve the integration constant by requiring zero deflection
        # at the left support - the right one comes automatically
        # TODO [SR, HS] this is specific to 3 point bending - generalize
        #           for other loading conditions.
        #           HS: I guess this works for 4pb too (for symmetric beams)
        # if self.beam_design.beam_conf_name != BoundaryConfig.FIXED_AND_ROLLER_SUPPORT_DIST_LOAD \
        #         and self.beam_design.beam_conf_name != BoundaryConfig.CANTILEVER_DIST_LOAD:
        #     w_x += w_x[0]

        return w_x

    theta_max = tr.Float(1)

    F_max = tr.Property(Float)
    ''''
    Identify the ultimate limit state based on the maximum moment capacity
    of the cross section.
    '''
    def _get_F_max(self):
        # specific to 3pt bending  - equation should be provided by the
        # BoundaryCondition class - corresponding to the load configuration.
        # TODO [SR, HS] generalize the following and use something better than M_I[-1] and M_I[0]
        #           for other loading conditions.
        M_I, kappa_I = self.mc.inv_M_kappa
        if self.beam_design.beam_conf_name == BoundaryConfig.THREE_PB:
            F_max = 4 * M_I[-1] / self.beam_design.L
        elif self.beam_design.beam_conf_name == BoundaryConfig.FOUR_PB:
            load_distance = self.beam_design.beam_conf_name.first_load_distance
            if load_distance == 0:
                load_distance = self.beam_design.L / 3
            F_max = M_I[-1] / load_distance
        elif self.beam_design.beam_conf_name == BoundaryConfig.SIMPLE_BEAM_DIST_LOAD:
            F_max = 8 * M_I[-1] / self.beam_design.L**2
        elif self.beam_design.beam_conf_name == BoundaryConfig.THREE_SPAN_DIST_LOAD:
            # maximum negative moment M_I[0]
            F_max = 10 * M_I[0] / self.beam_design.L**2  # max moment is in the 2nd support
        elif self.beam_design.beam_conf_name == BoundaryConfig.FIXED_SUPPORT_DIST_LOAD:
            F_max = 24 * M_I[-1] / self.beam_design.L**2 # max moment is in the span middle
        elif self.beam_design.beam_conf_name == BoundaryConfig.FIXED_AND_ROLLER_SUPPORT_DIST_LOAD:
            # maximum negative moment M_I[0]
            F_max = 8 * M_I[0] / self.beam_design.L**2 # max moment is in fixed support
        elif self.beam_design.beam_conf_name == BoundaryConfig.CANTILEVER_DIST_LOAD:
            # maximum negative moment M_I[0]
            F_max = 2 * M_I[0] / self.beam_design.L**2
        return abs(F_max)
    

    # def run(self):
    #     F_arr = np.linspace(0, self.F_max, self.n_load_steps)
    #     w_list = []
    #     original_F = self.beam_design.F
    #     for F in F_arr:
    #         if F == 0:
    #             w_list.append(0)
    #         else:
    #             self.beam_design.F = -F
    #             # Append the maximum deflection value that corresponds to the new load (F)
    #             w_list.append(np.fabs(np.min(self.get_w_x())))
    #     self.beam_design.F = original_F
    #     return F_arr, np.array(w_list)
    #
    # def reset(self):
    #     self.theta_F = 0

    F_max_old = Float

    def get_Fw(self):
        F_max = self.F_max
        F_arr = np.linspace(0, F_max, self.n_load_steps)
        w_list = []
        # @todo [SR,RC]: separate the slider theta_F from the calculation
        #                of the datapoints load deflection curve.
        #                use broadcasting in the functions
        #                get_M_x(x[:,np.newaxis], F[np.newaxis,:] and
        #                in get_Q_x, get_kappa_x, get_w_x, get_phi_x, get_w_x
        #                then, the browsing through the history is done within
        #                the two dimensional array of and now loop over theta is
        #                neeeded then. Theta works just as a slider - as originally
        #                introduced.
        original_F = self.beam_design.F
        for F in F_arr:
            if F == 0:
                w_list.append(0)
            else:
                self.beam_design.F = -F
                # Append the maximum deflection value that corresponds to the new load (F)
                w_list.append(np.max(np.fabs(self.get_w_x())))
        if self.F_max_old == F_max:
            self.beam_design.F = original_F
        self.F_max_old = F_max
        return F_arr, np.array(w_list)
    
    # def get_Fw_inx(self, inx):
    #     F_max = self.F_max
    #     F_arr = np.linspace(0, F_max, self.n_load_steps)
    #     w_list = []
    #     # @todo [SR,RC]: separate the slider theta_F from the calculation
    #     #                of the datapoints load deflection curve.
    #     #                use broadcasting in the functions
    #     #                get_M_x(x[:,np.newaxis], F[np.newaxis,:] and
    #     #                in get_Q_x, get_kappa_x, get_w_x, get_phi_x, get_w_x
    #     #                then, the browsing through the history is done within
    #     #                the two dimensional array of and now loop over theta is
    #     #                neeeded then. Theta works just as a slider - as originally
    #     #                introduced.
    #     original_F = self.beam_design.F
    #     for F in F_arr:
    #         if F == 0:
    #             w_list.append(0)
    #         else:
    #             self.beam_design.F = -F
    #             # Append the maximum deflection value that corresponds to the new load (F)
    #             w_list.append(np.fabs(self.get_w_x()[inx]))
    #     if self.F_max_old == F_max:
    #         self.beam_design.F = original_F
    #     self.F_max_old = F_max
    #     return F_arr, np.array(w_list)


    def subplots(self, fig):
        gs = gridspec.GridSpec(1, 2, figure=fig, width_ratios=[0.7, 0.3])

        # ax2, ax3 = fig.subplots(1, 2)
        ax_w = fig.add_subplot(gs[0, 0])
        ax_k = ax_w.twinx()
        ax_Fw = fig.add_subplot(gs[0, 1])
        return ax_w, ax_k, ax_Fw

    def update_plot(self, axes):
        ax_w, ax_k, ax_Fw = axes
        self.plot_fw_with_fmax(ax_Fw)
        self.plot_curvature_along_beam(ax_k)
        self.plot_displacement_along_beam(ax_w)
        mpl_align_yaxis_to_zero(ax_w, ax_k)
        mpl_show_one_legend_for_twin_axes(ax_w, ax_k)

    def plot_fw_with_fmax(self, ax_Fw):
        self.plot_fw(ax_Fw)
        current_F = abs(self.F_scale * self.beam_design.F)
        ax_Fw.axhline(y=current_F, color='r')
        ax_Fw.annotate('F = {} kN'.format(round(current_F,2)), xy=(0, current_F), color='r')

    def plot_curvature_along_beam(self, ax_k):
        x = self.beam_design.x
        kappa_x = self.get_kappa_x()
        ax_k.plot(x, kappa_x, color='black', label='$\kappa [\mathrm{mm}^{-1}]$')
        ax_k.plot(x, self.get_phi_x()/1000, color='green', label='$\phi [rad]$')
        ax_k.fill_between(x, 0, kappa_x, color='gray', alpha=0.1)
        ax_k.invert_yaxis()
        ax_k.set_ylabel(r'$\kappa [\mathrm{mm}^{-1}]$')
        ax_k.set_xlabel(r'$x$')

    def plot_displacement_along_beam(self, ax_w):
        x = self.beam_design.x
        w_x = self.get_w_x()
        ax_w.plot(x, w_x, color='blue', label='$w$ [mm]')
        ax_w.fill_between(x, 0, w_x, color='blue', alpha=0.1)
        ax_w.set_ylabel(r'$w [\mathrm{mm}]$')


    F_scale = tr.Float(1/1000)

    def plot_fw(self, ax_Fw):
        # TODO: expensive calculations for all displacements are running with each plot update to produce new
        #  load-displacement curve, this shouldn't be done for example when only the force has changed
        ax_Fw.set_xlabel(r'$w_\mathrm{max}$ [mm]')
        ax_Fw.set_ylabel(r'$F$ [kN]')
        F, w = self.get_Fw()
        ax_Fw.plot(w, self.F_scale * F,  label='sim deflection', lw=2)


class LoadDeflectionParamsStudy(ParametricStudy):

    def __init__(self, dp):
        self.dp = dp

    def plot(self, ax, param_name, value):
        ax.set_xlabel(r'$w_\mathrm{max}$ [mm]')
        ax.set_ylabel(r'$F$ [kN]')
        F, w = self.dp.get_Fw()
        ax.plot(w, self.dp.F_scale * F, label=param_name + '=' + str(value), lw=2)
        ax.set_title(param_name)
        ax.legend()


'''
Created on 26. 2. 2014

@author: Vancikv

Cross section from ex01 shown in a tree view.

Note that you can pass any tree node as root to the MxNTreeView
object. However, the standard case that allows for the whole
functionality to be used is the case when an instance of
UseCaseContainer is passed as root - as shown in examples 07-10.
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.view import \
    MxNTreeView

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCBar

bar = RLCBar(x=0.1, z=0.05, material='bar_d10')
'''Single steel reinforcement bar
'''

ge = MCSGeoRect(height=0.5, width=0.3)
'''Cross section geometry
'''

mcs = MatrixCrossSection(geo=ge,
                         n_cj=20,
                         material='default_mixture',
                         material_law='constant')
'''Matrix cross section
'''

cs = CrossSection(reinf=[bar],
                  matrix_cs=mcs,
                  eps_lo=0.002,
                  eps_up=-0.0033
                  )

csw = MxNTreeView(root=cs)
csw.configure_traits()



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Feb 16, 2010 by: rch

from traits.api import HasTraits, Interface

class IExRun( Interface ):
    '''Read the data from the directory
    '''



#-------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Apr 15, 2010 by: rch

from functools import reduce
import os
import pickle
import string

from traits.api import \
    HasTraits, HasStrictTraits, Dict, Str, Enum, Instance, Int, Type, \
    Directory, List, Property, Float, cached_property, Constant
from traitsui.api import \
    View, Item, Tabbed, VGroup, HGroup, ModelView, HSplit, VSplit, \
    CheckListEditor, EnumEditor, TableEditor, TabularEditor, Handler
from traitsui.menu import Action, CloseAction, HelpAction, Menu, \
    MenuBar, NoButtons, Separator, ToolBar
from traitsui.table_column import \
    ObjectColumn

from bmcs_beam.mxn.matresdev.db.simdb.simdb import simdb
from traitsui.tabular_adapter \
    import TabularAdapter
from traitsui.tabular_adapter \
    import TabularAdapter, AnITabularAdapter


class SimDBTableAdapter (TabularAdapter):

    columns = Property

    #---------------------------------------------------------------
    # EXTRACT FACTOR NAMES AS COLUMNS FOR TABLE EDITOR
    #-------------------------------------------------------------------
    def _get_columns(self):
        cols = [('index', 'index')]  # , ('key', 'dbkey') ]
        obj = self.object
        for field_name in obj.field_names:
            cols.append((field_name, field_name))
        return cols

    selection_mode = 'rows',

    font = 'Courier 10'
    alignment = 'right'
    odd_bg_color = 'lightblue'

    index_width = Float(50)

    index_text = Property

    def _get_index_text(self):
        return str(self.row)

    key_width = Float(120)
    key_text = Property

    def _get_key_text(self):
        factor_idx = self.column - 1
        value = self.object.instances[self.row, factor_idx]
        return str(value)


simdb_table_editor = TabularEditor(adapter=SimDBTableAdapter(),
                                   selected='selected_instance'
                                   )


class SimDBClass(HasTraits):
    '''Base class for instances storable as in SimDBClassExt'''

    key = Str()

    def insert(self):
        '''Let the object save itself
        '''
        self.db[self.key] = self

    def save(self):
        '''Let the object save itself
        '''
        self.db.save_item(self.key, self)

#-------------------------------------------------------------------------
# Class Extension - global persistent container of class instances
#-------------------------------------------------------------------------


class SimDBClassExt(HasStrictTraits):

    category = Enum('matdata', 'exdata')

    def _category_default(self):
        return 'matdata'

    path = List([])

    # dictionary of predefined instances - used for
    # debugging and early stages of class developmemnt.
    #
    klass = Type

    # Verbose mode either none, or upon write/save pickle file
    # or upon deletion of pickle files.
    verbose = Enum('none', 'io', 'del')

    classname = Property(depends_on='klass')

    @cached_property
    def _get_classname(self):
        return self.klass.__name__

    field_names = Property

    def _get_field_names(self):
        '''
        Get the dictionary of factors provided by the simulation model.

        The factors are identified by the factor_levels metadata in the trait
        definition. For example

        my_param = Float( 20, factor_levels = (0, 10, 6) )

        specifies a float factor with  the levels [0,2,4,6,8,10]
        '''
        traits = self.klass.class_traits(simdb=lambda x: x != None)
        return list(traits.keys())

    constants = Dict({})

    keyed_constants = Property(List)

    def _get_keyed_constants(self):
        for key, c in list(self.constants.items()):
            c.key = key
        return self.constants

    dirname = Str

    def _dirname_default(self):
        '''Name of the directory for the data of the class
        '''
        klass_dir = self.klass.__name__
        full_path = (simdb.simdb_dir, self.category) \
            + tuple(self.path) + (klass_dir,)
        path = os.path.join(*full_path)
        return path

    dir = Directory()

    def _dir_default(self):
        '''Directory for the data of the class
        '''
        # foolproof creation of the directory
        try:
            os.makedirs(self.dirname)
        except OSError:
            if os.path.exists(self.dirname):
                # We are nearly safe
                pass
            else:
                # There was an error on creation, so make sure we know about it
                raise
        return self.dirname

    instances = Dict

    def _instances_default(self):
        '''Read the content of the directory
        '''
        instances = {}
        for obj_file_name in os.listdir(self.dir):
            print(self.dir)
            # check to see whether the file is pickle or not
            path = os.path.join(self.dir, obj_file_name)
            if not os.path.isfile(path):
                continue
            obj_file = open(path, 'rb')
            key_list = obj_file_name.split('.')[:-1]
            key = reduce(lambda x, y: x + '.' + y, key_list)

            if self.verbose == 'io':
                print('%s.db: reading %s' % (self.klass.__name__, key))

            print(path)
            try:
                instances[key] = pickle.load(obj_file)
            except ImportError as e:
                print('file name %s' % obj_file)
                raise ImportError(e)

            # let the object know its key
            instances[key].key = key
            obj_file.close()
        return instances

    inst_list = Property

    def _get_inst_list(self):
        return list(self.keyed_constants.values()) + list(self.instances.values())

    selected_instance = Instance(SimDBClass)

    def _selected_instance_default(self):
        if len(self.inst_list) > 0:
            return self.inst_list[0]
        else:
            return None

    def keys(self):
        return list(self.keyed_constants.keys()) + list(self.instances.keys())

    def get(self, name, Missing):
        it = self.keyed_constants.get(name, Missing)
        if it == Missing:
            it = self.instances.get(name, Missing)
        return it

    def __setitem__(self, key, value):
        ''' Save the instance with the specified key.
        '''
        # check if the key corresponds to a constant
        # if yes, report an error

        if key in list(self.keys()):
            raise IndexError('an object with key %s already exists' % key)

        it = self.keyed_constants.get(key, None)
        if it:
            raise ValueError('attempting to change a constant %s' % key)
        else:
            self.save_item(key, value)
            # register the object in the memory as well
            self.instances[key] = value

    def save_item(self, key, value):
        for x in string.whitespace:
            key = key.replace(x, "_")
        # write to the database
        # value.key = key
        obj_file_name = os.path.join(self.dir, key + '.pickle')
        obj_file = open(obj_file_name, 'wb')

        if self.verbose == 'io':
            print('%s.db: writing %s' % (self.klass.__name__, key))

        pickle.dump(value, obj_file, protocol=0)  # slow text mode
        obj_file.close()

    def __getitem__(self, key):
        ''' Return the instance with the specified key.
        '''
        it = self.keyed_constants.get(key, None)
        if it == None:
            it = self.instances.get(key, None)
            if it == None:
                raise ValueError('No database object with the key %s for class %s' % (
                    key, self.classname))
        return it

    def __delitem__(self, key):
        # check if the key corresponds to a constant
        # if yes, report an error
        it = self.keyed_constants.get(key, None)
        if it:
            raise ValueError('attempting to delete a constant %s' % key)
        else:
            for x in string.whitespace:
                key = key.replace(x, "_")
            # write to the database
            obj_file_name = os.path.join(self.dir, key + '.pickle')
            if os.path.exists(obj_file_name):
                os.remove(obj_file_name)
            del self.instances[key]

    def delete_instances(self):
        for key in list(self.instances.keys()):
            self.__delitem__(key)

    #-------------------------------------------------------------------------
    # VIEW
    #-------------------------------------------------------------------------

    traits_view = View(
        HSplit(
            VSplit(
                VGroup(
                    HGroup(Item('classname',
                                style='readonly',
                                label='database extension class')
                           ),
                    Item('inst_list',
                         editor=simdb_table_editor,
                         show_label=False,
                         style='custom'),
                    label='database table',
                    id='simbd.table.instances',
                    dock='tab',
                ),
                id='simdb.table.left',
            ),
            VGroup(
                VGroup(
                    Item('selected_instance@',
                         resizable=True,
                         show_label=False),
                    label='instance',
                    id='simdb.table.instance',
                    dock='tab',
                    scrollable=True,
                ),
                id='simdb.table.right',
                layout='split',
                label='selected instance',
                dock='tab',
            ),
            id='simdb.table.splitter',
        ),
        id='simdb.table',
        dock='tab',
        resizable=True,
        buttons=['OK', 'Cancel'],
        height=0.8, width=0.8,
    )



'''
Created on 6. 2. 2014

@author: Vancikv

Example of cross section assembly - I-shaped concrete
cross section with combined reinforcement of textile
layers and steel bars
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoI

from bmcs_beam.mxn.reinf_layout import \
    RLCTexLayer, RLCBar

from matplotlib.figure import \
    Figure

from matplotlib.backends.backend_agg import \
    FigureCanvasAgg

from bmcs_beam.mxn.utils import \
    get_outfile

import numpy as np

ge = MCSGeoI(height=0.4, height_up=0.05, width_up=0.25,
             height_lo=0.05, width_lo=0.35, width_st=0.05)
mcs = MatrixCrossSection(geo=ge,
                         n_cj=20,
                         material='default_mixture',
                         material_law='constant')
'''Cross section geometry + matrix
'''

bar1 = RLCBar(x=0.025, z=0.025, material='bar_d10',
              material_law='steel')
bar2 = RLCBar(x=0.125, z=0.025, material='bar_d10',
              material_law='steel')
bar3 = RLCBar(x=0.225, z=0.025, material='bar_d10',
              material_law='steel')
bar4 = RLCBar(x=0.325, z=0.025, material='bar_d10',
              material_law='steel')
'''Four steel reinforcement bars in lower flange
'''

tl1 = RLCTexLayer(z_coord=0.39, material='default_fabric',
                  material_law='fbm')
tl2 = RLCTexLayer(z_coord=0.01, material='default_fabric',
                  material_law='fbm')
'''Two layers of textile reinforcement
'''

cs = CrossSection(reinf=[tl1, tl2, bar1, bar2, bar3, bar4],
                         matrix_cs=mcs,
                         eps_lo=0.002,
                         eps_up=-0.0033,
                         )

print('normal force', cs.N)
print('moment', cs.M)

fig = Figure(figsize=(10, 7), dpi=80, facecolor='white')
canvas = FigureCanvasAgg(fig)
ax = fig.add_subplot(1, 1, 1)
cs.plot_geometry(ax)

canvas.print_figure(get_outfile(folder_name='.mxn',
                                file_name='ex05.png'))



from numpy import array, linspace, trapz, arange
from traits.api import Array, Bool, Callable, Enum, Float, Event, HasTraits, \
    Instance, Int, Trait, ToolbarButton, Button, on_trait_change, \
    Property, cached_property, List
from traitsui.api import Item, View, Group, Handler, HGroup

from traitsui.menu import NoButtons, OKButton, CancelButton, Action, CloseAction, Menu, \
    MenuBar, Separator

from .mfn_line import MFnLineArray

import time
import math


class MFnMultiLine(HasTraits):

    # Public Traits
    lines = List(MFnLineArray)

    xdata = Property(List(Array))

    def _get_xdata(self):
        return [mfn.xdata for mfn in self.lines]

    ydata = Property(List(Array))

    def _get_ydata(self):
        return [mfn.ydata for mfn in self.lines]

    data_changed = Event



from numbers import Number

import numpy as np
import sympy as sp
import traits.api as tr
from matplotlib.patches import PathPatch
from matplotlib.path import Path
from sympy.physics.continuum_mechanics.beam import Beam
import matplotlib.patches as mpatches

import enum

# class BoundaryConfigSettings:
#     pass

class BoundaryConfig(enum.Enum):
    # The following should be provided as sub classes with possible settings change and get moment ability
    THREE_PB, FOUR_PB, SIMPLE_BEAM_DIST_LOAD, FIXED_SUPPORT_DIST_LOAD, FIXED_AND_ROLLER_SUPPORT_DIST_LOAD, CANTILEVER_DIST_LOAD, THREE_SPAN_DIST_LOAD, THREE_PB_FIXED_SUPPORT, SINGLE_MOMENT = range(9)

    # TODO [HS] This is not initialized when any of the above enums are created, fix this
    first_load_distance = 0

    # settings = BoundaryConfigSettings()


class BoundaryConditions(tr.HasTraits):
    # loads = tr.List
    # supports = tr.List
    name = 'BoundaryConditions'

    @staticmethod
    def get_configured_beam(L, F, config):
        # input beam should have a numerical length and has E and I as symbols, and have predefined supports
        R1, R2, R3, R4, M1, M2 = sp.symbols('R1, R2, R3, R4, M1, M2')
        E, I = sp.symbols('E, I')
        beam = Beam(L, 1, 1)
        if config == BoundaryConfig.THREE_PB:
            # 3 point bending example
            beam.apply_load(R1, 0, -1)
            beam.apply_load(R2, L, -1)
            beam.apply_load(F, L/2, -1)
            beam.bc_deflection = [(0, 0), (L, 0)]

        elif config == BoundaryConfig.FOUR_PB:
            # 4 point bending example
            load_distance = config.first_load_distance
            if load_distance == 0:
                load_distance = L/3
            beam.apply_load(R1, 0, -1)
            beam.apply_load(R2, L, -1)
            beam.apply_load(F, load_distance, -1)
            beam.apply_load(F, L - load_distance, -1)
            beam.bc_deflection = [(0, 0), (L, 0)]
            
        elif config == BoundaryConfig.SIMPLE_BEAM_DIST_LOAD:
            # distributed load simple beam example
            beam.apply_load(R1, 0, -1)
            beam.apply_load(R2, L, -1)
            beam.apply_load(F, 0, 0)
            beam.bc_deflection = [(0, 0), (L, 0)]

        elif config == BoundaryConfig.THREE_SPAN_DIST_LOAD:
            # 3 span distributed load example
            beam.apply_load(R1, 0, -1)
            beam.apply_load(R2, L / 3, -1)
            beam.apply_load(R3, 2 * L / 3, -1)
            beam.apply_load(R4, L, -1)
            beam.apply_load(F, 0, 0)
            beam.bc_deflection = [(0, 0), (L / 3, 0), (2 * L / 3, 0), (L, 0)]

        elif config == BoundaryConfig.THREE_PB_FIXED_SUPPORT:
            # fixed support example
            beam.apply_load(R1, 0, -1)
            beam.apply_load(M1, 0, -2)
            beam.apply_load(R2, L, -1)
            beam.apply_load(M2, L, -2)
            beam.apply_load(F, L / 2, -1)
            beam.bc_deflection = [(0, 0), (L, 0)]
            beam.bc_slope = [(0, 0), (L, 0)]

        elif config == BoundaryConfig.SINGLE_MOMENT:
            # single moment example
            beam.apply_load(R1, 0, -1)
            beam.apply_load(R2, L, -1)
            beam.apply_load(-F, L / 2, -2)
            beam.bc_deflection = [(0, 0), (L, 0)]

        elif config == BoundaryConfig.FIXED_SUPPORT_DIST_LOAD:
            beam.apply_load(R1, 0, -1)
            beam.apply_load(M1, 0, -2)
            beam.apply_load(R2, L, -1)
            beam.apply_load(M2, L, -2)
            beam.apply_load(F, 0, 0)
            beam.bc_deflection = [(0, 0), (L, 0)]
            beam.bc_slope = [(0, 0), (L, 0)]

        elif config == BoundaryConfig.FIXED_AND_ROLLER_SUPPORT_DIST_LOAD:
            beam.apply_load(R1, 0, -1)
            beam.apply_load(R2, L, -1)
            beam.apply_load(M2, L, -2)
            beam.apply_load(F, 0, 0)
            beam.bc_deflection = [(0, 0), (L, 0)]
            beam.bc_slope = [(L, 0)]

        elif config == BoundaryConfig.CANTILEVER_DIST_LOAD:
            beam.apply_load(R1, 0, -1)
            beam.apply_load(M1, 0, -2)
            beam.apply_load(F, 0, 0)
            beam.bc_deflection = [(0, 0)]
            beam.bc_slope = [(L, 0)]

        return beam

    @staticmethod
    def plot(ax, beam):
        L = float(beam.length)
        # ax.annotate('L = {} mm'.format(np.round(L), 0), xy=(L / 2, 5), color='black')

        # Beam
        ax.plot([0, beam.length], [0, 0], linewidth=2, color='black')

        # supports
        vertices = []
        codes = []

        support_width = 0.02 * beam.length
        support_line_distance = 1.3 * support_width

        for i, deflection in enumerate(beam.bc_deflection):
            x_loc, def_value = deflection

            # Makeing sure it's a support (deflection value is 0)
            if def_value == 0:
                # Draw the triangle of the support
                codes += [Path.MOVETO, Path.LINETO, Path.LINETO, Path.CLOSEPOLY]
                vertices += [(x_loc, 0),
                             (x_loc + support_width / 2, -support_width),
                             (x_loc - support_width / 2, -support_width),
                             (x_loc, 0)]

                # Add line below the triangle for other supports
                if i > 0:
                    codes += [Path.MOVETO] + [Path.LINETO] + [Path.CLOSEPOLY]
                    vertices += [(x_loc - support_width / 2, -support_line_distance),
                                 (x_loc + support_width / 2, -support_line_distance),
                                 (x_loc + support_width / 2, -support_line_distance)]

        # for i in range(0, len((self.get_supports_loc()[1]))):
        #     codes += [Path.MOVETO] + [Path.LINETO] * 3 + [Path.CLOSEPOLY]
        #     vertices += [(self.get_supports_loc()[1][i] - L * (self.G_adj), -L * self.G_adj * 1.2),
        #                  (self.get_supports_loc()[1][i] - L * (self.G_adj), self.H + L * self.G_adj * 1.2),
        #                  (self.get_supports_loc()[1][i] + L * (self.G_adj), self.H + L * self.G_adj * 1.2),
        #                  (self.get_supports_loc()[1][i] + L * (self.G_adj), -L * self.G_adj * 1.2),
        #                  (self.get_supports_loc()[1][i] - L * (self.G_adj), -L * self.G_adj * 1.2)]

        vertices = np.array(vertices, float)
        path = Path(vertices, codes)
        pathpatch = PathPatch(path, facecolor='black', edgecolor='black')
        ax.add_patch(pathpatch)

        # loads
        load = beam.applied_loads
        value = [0]
        pos = [0]
        type = [0]

        # reading the value and the position of external loads
        for load in beam.applied_loads:
            load_value = load[0]
            if isinstance(load_value, Number):
                load_value = float(load[0])
                load_position = float(load[1])
                load_type = float(load[2])

                x_tail = load_position
                x_head = load_position

                # Point load
                if load_type == -1:
                    y_tail = L / 20
                    y_head = 2.
                    xy_annotate = (x_tail, y_tail)

                    if load_value == abs(load_value):
                        y_tail = 2.
                        y_head = L / 20
                        xy_annotate = (x_head, y_head)

                    arrow = mpatches.FancyArrowPatch((x_tail, y_tail), (x_head, y_head),
                                                     color='blue', mutation_scale=L / 500)
                    ax.annotate('{} KN'.format(-round(load_value / 1000., 2)), xy=xy_annotate, color='black')
                    ax.add_patch(arrow)

        # for i in range(0, len(load)):
        #     if load[i][0] == -self.F or load[i][0] == self.F:
        #         value[0] = (load[i][0])
        #         pos[0] = (load[i][1])
        #         load_dic = dict(zip(value, pos))
        #         load_ = sp.lambdify((self.F, L), load_dic)
        #         Load_x = load_(self.f, L)
        #
        #         # load arrow parameters
        #         x_tail = (list(Load_x.values())[0])
        #         x_head = (list(Load_x.values())[0])
        #
        #         # moment
        #         if load[i][2] == -2:
        #
        #             if load[i][0] == -self.F:
        #
        #                 ax.plot([(list(Load_x.values())[0])], [(self.H / 2)],
        #                         marker=r'$\circlearrowleft$', ms=self.H / 5)
        #
        #             else:
        #
        #                 ax.plot([(list(Load_x.values())[0])], [(self.H / 2)],
        #                         marker=r'$\circlearrowright$', ms=self.H / 5)
        #
        #             ax.annotate('{} KN.mm'.format(np.round(self.f / 1000), 0),
        #                         xy=(list(Load_x.values())[0], self.H * 1.4), color='blue')
        #
        #         # point load
        #         elif load[i][2] == -1:
        #
        #             if load[i][0] == -self.F:
        #
        #                 y_tail = L / 20 + self.H
        #                 y_head = 0 + self.H
        #                 dy = y_head + x_head / 10
        #
        #                 arrow = mpatches.FancyArrowPatch((x_tail, y_tail), (x_head, y_head),
        #                                                  color='blue', mutation_scale=L / 500)
        #                 ax.annotate('{} KN'.format(np.round(self.f / 1000), 0),
        #                             xy=(x_tail, y_tail), color='black')
        #                 ax.add_patch(arrow)
        #
        #             else:
        #
        #                 y_tail = 0 + self.H
        #                 y_head = L / 20 + self.H
        #                 dy = y_head + x_head / 10
        #
        #                 arrow = mpatches.FancyArrowPatch((x_tail, y_tail), (x_head, y_head),
        #                                                  color='blue', mutation_scale=L / 500)
        #                 ax.annotate('{} KN'.format(np.round(self.f / 1000), 0),
        #                             xy=(x_head, y_head), color='black')
        #                 ax.add_patch(arrow)
        #
        #         else:
        #
        #             if load[i][0] == -self.F:
        #                 y_tail = L / 20 + self.H
        #                 y_head = 0 + self.H
        #                 dy = y_head + x_head / 10
        #
        #                 # distributed load
        #                 l_step = 0
        #                 while l_step <= L:
        #                     x_tail = (list(Load_x.values())[0]) + l_step
        #                     y_tail = L / 20 + self.H
        #                     x_head = (list(Load_x.values())[0]) + l_step
        #                     y_head = 0 + self.H
        #                     dy = y_head + x_head / 10
        #                     l_step += L / 10
        #                     arrow = mpatches.FancyArrowPatch((x_tail, y_tail), (x_head, y_head),
        #                                                      color='blue', mutation_scale=L / 500)
        #                     ax.add_patch(arrow)
        #
        #                 ax.annotate('{} KN'.format(np.round(self.f / 1000), 0),
        #                             xy=(L / 2, y_tail * 1.1), color='black')
        #                 ax.plot([0, L], [y_tail, y_tail], color='blue')
        #
        #             else:
        #                 y_tail = 0 + self.H
        #                 y_head = L / 20 + self.H
        #                 dy = y_head + x_head / 10
        #
        #                 # distributed load
        #                 l_step = 0
        #                 while l_step <= L:
        #                     x_tail = (list(Load_x.values())[0]) + l_step
        #                     y_tail = 0 + self.H
        #                     x_head = (list(Load_x.values())[0]) + l_step
        #                     y_head = L / 20 + self.H
        #                     dy = y_head + x_head / 10
        #                     l_step += L / 10
        #                     arrow = mpatches.FancyArrowPatch((x_tail, y_tail), (x_head, y_head),
        #                                                      color='blue', mutation_scale=L / 500)
        #                     ax.add_patch(arrow)
        #
        #                 ax.annotate('{} KN'.format(np.round(self.f / 1000), 0),
        #                             xy=(L / 2, y_head * 1.1), color='black')
        #                 ax.plot([0, L], [y_head, y_head], color='blue')

        ax.axis('equal')
        ax.autoscale(tight=True)


# class BoundaryCondition(tr.HasTraits):
#     x_loc = tr.Float
#
#
# class Support(BoundaryCondition):
#     constrained_directions = tr.Tuple(0, 1)  # free on x and constrained on y
#
#
# class Load(BoundaryCondition):
#     value = tr.Float



'''
Created on Apr 11, 2016

@author: rch
'''
import numpy as np


def iron_measured_curve(y_asc, x_asc, t_asc, jump_rtol=0.0001):
    ''''Force curve with removes jumps in the stress strain curve
    due to sliding in the buttstrap clamping. The small unloading/loading
    branches are removed from the data and the smoothed curve is
    determined based on the remaining data.
    In order to smoothen out the effect of the jump the pieces of the
    F-w-curve that contain jumps in the force (=unloading/reloading) path
    are removed from the data in a range up to the double range of the jump,
    so that the smoothing does not change the shape of the F-w- curve.
    '''
    F_asc = y_asc
    eps_asc = x_asc
    time_asc = t_asc

    # get the differences of the force values between two adjacent
    # time steps
    #
    jump_arr = F_asc[1:] - F_asc[0:-1]

    # determine the criteria for a jump
    # based on the data range and the specified tolerances:
    #
    jump_crit = jump_rtol * F_asc[-1]

    # get the indices of the measurement data at which a
    # force jump exceeds (last step before the jump) the defined tolerance
    # criteria
    # i.e. negative jump that exceeds the defined tolerance magnitude
    #
    jump_idx_arr = np.where(jump_arr < -jump_crit)[0] - 1

    # index of the measurement data where the force reaches
    # the same magnitude before the sudden value drop due to the jump
    #
    jump_idx2_arr = np.zeros_like(jump_idx_arr)

    # amount of indices between the sudden value drop of the force and
    # the reloading to the same load level; delta value indicate
    # the strain range that will be removed in order to smoothen out
    # the influence of the jump in the force curve
    #
    delta_jump_idx_arr = np.zeros_like(jump_idx_arr)

    # search at which index the force reaches its old value before
    # the jump again check that this value is index wise and time
    # wise reached after the jump occurred
    #
    for n_idx, jump_idx in enumerate(jump_idx_arr):
        delta_F = F_asc - F_asc[jump_idx]
        delta_eps = eps_asc - eps_asc[jump_idx]
        delta_t = time_asc - time_asc[jump_idx]
        bool_arr_F = delta_F > 0.
        bool_arr_eps = delta_eps > 0.
        bool_arr_t = delta_t > 0.
        bool_arr = bool_arr_F * bool_arr_eps * bool_arr_t
        try:
            jump_idx2 = np.where(bool_arr)[0][1]
        except:
            break
        delta_jump_idx = jump_idx2 - jump_idx
        jump_idx2_arr[n_idx] = jump_idx2
        delta_jump_idx_arr[n_idx] = delta_jump_idx

    # remove jumps from the jump index when a succeeding jump still
    # lays within the influence range of an earlier jump
    # this can happen when jumps occur within the re-mounting
    # branch of the force
    #
    remove_idx = []
    for i in range(jump_idx2_arr.shape[0] - 1):
        if np.any(jump_idx2_arr[:i + 1] > jump_idx2_arr[i + 1]):
            remove_idx += [i + 1]

    jump_idx_arr = np.delete(jump_idx_arr, remove_idx)
    jump_idx2_arr = np.delete(jump_idx2_arr, remove_idx)
    delta_jump_idx_arr = np.delete(delta_jump_idx_arr, remove_idx)

    # specify the factor with whom the index delta range of a jump
    # (i.e. displacement range of the jump)
    # is multiplied, i.e. up to which index the values of the
    # F-w- curve are removed
    #
    jump_smooth_fact = 2

    # remove the values of the curve within the jump and the neighboring
    # region
    #
    F_asc_ironed_list = []
    eps_asc_ironed_list = []

    max_stress_idx = len(y_asc)
    jump_idx_arr_ = np.hstack(
        [np.array([0.]), jump_idx_arr, np.array([max_stress_idx])])
    delta_jump_idx_arr_ = np.hstack(
        [np.array([0]), delta_jump_idx_arr, np.array([0])])

    for i in range(jump_idx_arr_.shape[0] - 1):
        F_asc_ironed_list += \
            [F_asc[jump_idx_arr_[i] +
                   jump_smooth_fact *
                   delta_jump_idx_arr_[i]: jump_idx_arr_[i + 1]]]
        eps_asc_ironed_list += \
            [eps_asc[jump_idx_arr_[i] +
                     jump_smooth_fact *
                     delta_jump_idx_arr_[i]: jump_idx_arr_[i + 1]]]

    # remove the values of the curve within the jump
    # and the neighboring region
    #
    F_asc_ironed = np.hstack(F_asc_ironed_list)
    eps_asc_ironed = np.hstack(eps_asc_ironed_list)
    return F_asc_ironed, eps_asc_ironed



'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Property, cached_property

import numpy as np

from bmcs_beam.mxn.mfn import MFnLineArray

import sympy as sp

from .matrix_law_base import \
    MatrixLawBase


class MatrixLawBlock(MatrixLawBase):

    '''Effective crack bridge Law with linear elastic response.'''

    #-----------------------------
    #
    # for simplified constant stress-strain-diagram of the concrete (EC2)
    #-----------------------------
    mfn = Property(depends_on='+input')

    @cached_property
    def _get_mfn(self):
        '''simplified constant stress-strain-diagram of the concrete (EC2)
        '''
        # (for standard concrete)
        f_ck = self.f_ck + 8.
        if f_ck <= 50:
            lamda = 0.8
            eta = 1.0
            eps_cu3 = self.eps_c_u
        # (for high strength concrete)
        #
        else:
            eta = 1.0 - (f_ck / 50.) / 200.
        # factor [-] to calculate the height of the compressive zone
            lamda = 0.8 - (f_ck - 50.) / 400.
            eps_cu3 = (2.6 + 35. * ((90. - f_ck) / 100) ** 4.) / 1000.

        xdata = np.hstack(
            [0., (1. - lamda) * eps_cu3 - 0.00001, (1 - lamda) * eps_cu3, eps_cu3])
        ydata = np.hstack([0., 0., eta * (f_ck), eta * (f_ck), ])

        return MFnLineArray(xdata=xdata, ydata=ydata)



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Created on May 25, 2009 by Rostislav Chudoba
#
#-------------------------------------------------------------------------------

# Enthought library imports
from traits.api import \
    Enum, false, Str, Range, Tuple, Bool, Trait, Int, Any, Property, Instance, HasPrivateTraits
from traitsui.api import  Item, UI
from etsproxy.traits.ui.menu import Action, ToolBar, Menu
from etsproxy.traits.ui.wx.editor import Editor
from etsproxy.traits.ui.basic_editor_factory import BasicEditorFactory

from etsproxy.enable.api import black_color_trait, LineStyle, ColorTrait, white_color_trait, Window, Component
from etsproxy.kiva.traits.kiva_font_trait import KivaFont
from etsproxy.kiva.backend_image import GraphicsContext

from etsproxy.chaco.example_support import COLOR_PALETTE
from etsproxy.chaco.api import \
    Plot, ArrayPlotData, create_line_plot, add_default_axes, PlotLabel, VPlotContainer,\
    add_default_grids, OverlayPlotContainer, PlotAxis, Legend, ArrayPlotData, ArrayDataSource, \
    create_scatter_plot, Legend, PlotComponent, PlotGraphicsContext, DataRange1D, LinePlot, LinearMapper, \
    Plot, ArrayPlotData
from etsproxy.chaco.tools.api import \
                                PanTool, RectZoomTool, SimpleZoom, \
                                LegendTool, TraitsTool, DragZoom, \
                                BroadcasterTool, ZoomTool, LegendTool

from etsproxy.pyface.api import FileDialog, OK, ImageResource
from etsproxy.pyface.image_resource import ImageResource

from .mfn_plot_adapter import MFnPlotAdapter

from scipy.special import jn
import wx, csv #, sys

from numpy import *

USE_DATA_UPDATE = 1
WILDCARD = "Saved plots (*.png)|*.png|"\
           "All files (*.*)|*.*"

# Range for the height and width for the plot widget.
PlotSize = Range( 50, 1000, 180 )

class _MFnChacoEditor ( Editor ):
    """ Traits UI editor for displaying trait values in a MFnLine.
    """
  
    # TableEditorToolbar associated with the editor:
    toolbar = Any
    # The Traits UI associated with the function editor toolbar:
    #  toolbar_ui = Instance( UI )

    # adjustable parameters
    adapter = Instance( MFnPlotAdapter )
    
    splot = Instance(Plot)
    line_plot = Instance(LinePlot)
    
    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    plot_container = Instance( OverlayPlotContainer )
    def _plot_container_default( self ):
        container = OverlayPlotContainer( padding = 50, fill_padding = False,
                                  bgcolor = self.adapter.bgcolor, use_backbuffer=True)

        return container
    
    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        
    def init ( self, parent ):

        factory = self.factory
        self.adapter = factory.adapter

        self.control = self._create_canvas(parent)

        # Register the update listener
        #
        self.value.on_trait_change( self.update_editor, 'data_changed' )

    def update_editor( self):
        c = self.plot_container
        c.remove( *c.components )
        self._refresh_container()        

    def _create_canvas( self, parent ):
        '''Create canvas for chaco plots
        '''
        panel = wx.Panel( parent, -1, style=wx.CLIP_CHILDREN )
        
        container_panel = Window( panel, component = self.plot_container )
        
        sizer = wx.BoxSizer( wx.VERTICAL )
        sizer.Add( wx.StaticLine( parent, -1, style = wx.LI_HORIZONTAL ), 0,
                   wx.EXPAND | wx.BOTTOM, 5 )
        sizer.Add( self._create_toolbar( panel ), 0, wx.EXPAND )
        sizer.Add( container_panel.control, 1, wx.EXPAND )

        sizer.Add( wx.StaticLine( parent, -1, style = wx.LI_HORIZONTAL ), 0,
                   wx.EXPAND | wx.BOTTOM, 5 )

        panel.SetSizer(sizer)

        return panel

    def _refresh_container( self ):
        ''' rebuild the container for the current data
        '''
        broadcaster = BroadcasterTool()
    
        mfn_line = self.value
        ydata = transpose(mfn_line.ydata)
        
        adapter = self.adapter
        if adapter.var_x != '':
            # Get the x-label text from the object's trait var_x          
            label_x = getattr( self.object, adapter.var_x )
        else:
            # Get the x-label from the adapter          
            label_x = adapter.label_x
            
        if adapter.var_y  != '':
            label_y = getattr( self.object, adapter.var_y )
        else:
            label_y = adapter.label_y
        
        index = ArrayDataSource(mfn_line.xdata)
        index_range = DataRange1D()
        index_range.add(index)
        index_mapper = LinearMapper(range=index_range)
            
        value_range = DataRange1D(  low_setting = 0.0  )
        
        colors = [] 
        colors = adapter.line_color #self.line_color_chaco.values() 

        styles = []
        styles = adapter.line_style #self.line_style_chaco.values()

        s_item = list(styles.items())
        
        color_chaco = []
        style_chaco = []
        c_index = 0  # loop for colors
        s_index = 0  # loop for styles
        i=0          # for colors and styles
        plots = {}   # for legend

        pd = ArrayPlotData(index = mfn_line.xdata)
        self.splot = Plot(pd)

        for vector in ydata[:]:

            if len(colors) == c_index:
                c_index = 0
            if len(styles) == s_index:
                s_index = 0
            
            style_name = s_item[s_index][0]    
            color_chaco.append(colors[c_index])
            style_chaco.append(style_name)
            c_index = c_index + 1
            s_index = s_index + 1
            
            y = ArrayDataSource(vector, sort_order="none")
                  
            value_range.add(y)
            value_mapper = LinearMapper(range=value_range)
            
            self.line_plot = LinePlot(index = index, value = y,
                                            index_mapper = index_mapper,
                                            value_mapper = value_mapper,
                                            color = color_chaco[i],  
                                            line_width = adapter.linewidth,
                                            edge_color = 'blue',
                                            border_visible = False,
                                            line_style = style_chaco[i])
        
            add_default_grids(self.line_plot)
            add_default_axes(self.line_plot, vtitle= label_y, htitle= label_x)
    
            self.plot_container.add(self.line_plot)
#            pan = PanTool(line_plot)
#            zoom = SimpleZoom(line_plot, tool_mode="box", always_on=False)
#            broadcaster.tools.append(pan)
#            broadcaster.tools.append(zoom)
    
            # Add the traits inspector tool to the container
            #
 #           self.plot_container.tools.append(TraitsTool( self.plot_container ))
            
            self.line_plot.tools.append(PanTool(self.line_plot))
            self.line_plot.overlays.append(ZoomTool(self.line_plot))

            # Legend
            lgnd = adapter.legend_labels[i]
            plots[lgnd] = self.line_plot

            # change the color of the curves
            i = i +1  

        legend = Legend(component=self.plot_container, padding=10, align="ul")
        legend.tools.append(LegendTool(legend, drag_button="right"))
        self.plot_container.overlays.append(legend)

        # Set the list of plots on the legend
        legend.plots = plots

        # Add the title at the top
        self.plot_container.overlays.append(PlotLabel(adapter.title,
                                  component=self.plot_container,
                                  font = "swiss 16",
                                  overlay_position="top"))

    #---------------------------------------------------------------------------
    #  Creates the table editing tool bar:
    #---------------------------------------------------------------------------

    def _create_toolbar ( self, parent ):
        """ Creates the table editing toolbar.
        """
        factory = self.factory
            
        panel = wx.Panel( parent, -1, style=wx.CLIP_CHILDREN )
        toolbar = MFnChacoEditorToolbar( parent = parent, editor = self )
        tb_sizer = wx.BoxSizer( wx.HORIZONTAL )
        panel.SetSizer(tb_sizer)

        self.toolbar = toolbar
        tb_sizer.Add( toolbar.control, 0 )
        tb_sizer.Add( ( 1, 1 ), 1, wx.EXPAND )

        return panel
    
    #---------------------------------------------------------------------------
    #  Handles the user requesting that columns not be sorted:
    #---------------------------------------------------------------------------
    def on_savedata ( self ):
        """ Handles the user requesting that the data of the function is to be saved.
        """
        import os
        dlg = FileDialog(parent = self.control, 
                         title = 'Export function data',
                         default_directory=os.getcwd(),
                         default_filename="", wildcard='*.csv',
                         action='save as')
        if dlg.open() == OK:
            path = dlg.path

            print("Saving data to", path, "...")
            try:
                vectors = []
                x_values = self.value.xdata
                y_values = self.value.ydata
                #savetxt( path, vstack( (x_values, y_values[:,0], y_values[:,1], y_values[:,2]) ).transpose() )
                
                print('y_values', y_values)
                y_values_tr = y_values.transpose()
                for vector in y_values_tr[:]:
                    vectors.append(vector)

                savetxt( path, vstack( (x_values, vectors) ).transpose() )
                    
            except:
                print("Error saving!")
                raise
            print("Plot saved.")
        return
    
    def on_savefig ( self ):
        """ Handles the user requesting that the image of the function is to be saved.
        """
        import os
        dlg = FileDialog(parent = self.control, 
                         title = 'Save as image',
                         default_directory=os.getcwd(),
                         default_filename="", wildcard=WILDCARD,
                         action='save as')
        if dlg.open() == OK:
            path = dlg.path

            print("Saving plot to", path, "...")
            try:
                # Now we create a canvas of the appropriate size and ask it to render
                # our component.  (If we wanted to display this plot in a window, we
                # would not need to create the graphics context ourselves; it would be
                # created for us by the window.)
                size=(650,400)
                gc = GraphicsContext(size)
                self.plot_container.draw(gc)
                gc.save(path)
            except:
                print("Error saving!")
                raise
            print("Plot saved.")
        return

#-------------------------------------------------------------------------------
#  'MFnChacoEditorToolbar' class:
#-------------------------------------------------------------------------------
class MFnChacoEditorToolbar ( HasPrivateTraits ):
    """ Toolbar displayed in table editors.
    """
    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # Do not sort columns:
    save_data = Instance( Action,
                        {'name':    'Save as data',
                         'tooltip': 'Save the function values',
                         'action':  'on_savedata',
                         'enabled': True,
                         'image':   ImageResource( 'add' ) } )

     # Move current object up one row:
    save_fig = Instance( Action,
                        { 'name':    'Save as fig',
                         'tooltip': 'Save as figure',
                         'action':  'on_savefig',
                         'enabled': True,
                         'image':   ImageResource( 'save' ) } )

    # The table editor that this is the toolbar for:
    editor = Instance( _MFnChacoEditor )

    # The toolbar control:
    control = Any

    #---------------------------------------------------------------------------
    #  Initializes the toolbar for a specified window:
    #---------------------------------------------------------------------------

    def __init__ ( self, parent = None, **traits ):
        super( MFnChacoEditorToolbar, self ).__init__( **traits )
        factory = self.editor.factory

        actions = [ self.save_data, self.save_fig ]
        toolbar = ToolBar( image_size      = ( 16, 16 ),
                           show_tool_names = False,
                           show_divider    = False,
                           *actions )
        self.control = toolbar.create_tool_bar( parent, self )
        self.control.SetBackgroundColour( parent.GetBackgroundColour() )

        # fixme: Why do we have to explictly set the size of the toolbar?
        #        Is there some method that needs to be called to do the
        #        layout?
        self.control.SetSize( wx.Size( 23 * len( actions ), 16 ) )

    #---------------------------------------------------------------------------
    #  PyFace/Traits menu/toolbar controller interface:
    #---------------------------------------------------------------------------

    #---------------------------------------------------------------------------
    #  Adds a menu item to the menu bar being constructed:
    #---------------------------------------------------------------------------

    def add_to_menu ( self, menu_item ):
        """ Adds a menu item to the menu bar being constructed.
        """
        pass

    #---------------------------------------------------------------------------
    #  Adds a tool bar item to the tool bar being constructed:
    #---------------------------------------------------------------------------

    def add_to_toolbar ( self, toolbar_item ):
        """ Adds a toolbar item to the too bar being constructed.
        """
        pass

    #---------------------------------------------------------------------------
    #  Returns whether the menu action should be defined in the user interface:
    #---------------------------------------------------------------------------

    def can_add_to_menu ( self, action ):
        """ Returns whether the action should be defined in the user interface.
        """
        return True

    #---------------------------------------------------------------------------
    #  Returns whether the toolbar action should be defined in the user
    #  interface:
    #---------------------------------------------------------------------------

    def can_add_to_toolbar ( self, action ):
        """ Returns whether the toolbar action should be defined in the user
            interface.
        """
        return True

    #---------------------------------------------------------------------------
    #  Performs the action described by a specified Action object:
    #---------------------------------------------------------------------------

    def perform ( self, action, action_event = None ):
        """ Performs the action described by a specified Action object.
        """
        getattr( self.editor, action.action )()


class MFnChacoEditor( BasicEditorFactory ):
    """ Editor factory for plot editors.
    """
    
    klass = _MFnChacoEditor
    
    adapter = Instance( MFnPlotAdapter )
    def _adapter_default(self):
        return MFnPlotAdapter()




from traits.api import \
    HasTraits, \
    Instance, Int, on_trait_change, Expression, Button
    
from traitsui.api import \
    Item, View, HGroup

from bmcs_beam.mxn.mfn.mfn_line.mfn_multiline import MFnMultiLine
from bmcs_beam.mxn.mfn import MFnLineArray
from bmcs_beam.mxn.mfn.mfn_line.mfn_matplotlib_multiline_editor import MFnMatplotlibEditor
from bmcs_beam.mxn.mfn.mfn_line.mfn_chaco_multiline_editor import MFnChacoEditor
from bmcs_beam.mxn.mfn.mfn_line.mfn_plot_adapter import MFnMultiPlotAdapter
from numpy import linspace, frompyfunc

a = MFnMultiPlotAdapter( title = 'Force vs Displacement',
                        label_x = 'x axis',
                        label_y = 'y axis',
                        mline_color = ['green','blue', 'red'], 
                        mline_style = ['solid', 'dashed', 'dotted'],
                        legend_labels = ('1','2','3'),
                        mline_width = [3,2,1])

class AnalyticalFunction( HasTraits ):
    
    No_of_all_curves = Int(3)
    expression1 = Expression('x**2', auto_set = False, enter_set = True )
    expression2 = Expression('x**2.3', auto_set = False, enter_set = True )
    expression3 = Expression('x**2.6', auto_set = False, enter_set = True )
    refresh = Button('redraw')
    def _refresh_fired(self):
        
        #creates an empty plot data container as a list of MFnLineArray classes
        self.mfn.lines = self.No_of_all_curves * [MFnLineArray()]
        
        xdata = linspace(0,10,100)
        fneval1 = frompyfunc( lambda x: eval( self.expression1 ), 1, 1 )
        fneval2 = frompyfunc( lambda x: eval( self.expression2 ), 1, 1 )
        fneval3 = frompyfunc( lambda x: eval( self.expression3 ), 1, 1 )
        self.mfn.lines[0] = MFnLineArray(xdata = xdata, ydata = fneval1( xdata ))
        self.mfn.lines[1] = MFnLineArray(xdata = xdata, ydata = fneval2( xdata ))
        self.mfn.lines[2] = MFnLineArray(xdata = xdata, ydata = fneval3( xdata ))
        self.mfn.data_changed = True
        
    mfn = Instance( MFnMultiLine )
    def _mfn_default( self ):
        return MFnMultiLine()
    
    @on_trait_change('expression' )
    def update_mfn(self):
        self._refresh_fired()
    
    view_mpl = View( HGroup( Item( 'expression1' ),
                             Item( 'expression2' ),
                             Item( 'expression3' ),
                             Item('refresh' ) ),
                 Item( 'mfn', editor = MFnMatplotlibEditor(  
                        adapter = a), show_label = False ),
                 resizable = True,
                 scrollable = True,
                 height = 0.5, width = 0.5
                    )
    
    view_chaco = View( HGroup( Item( 'expression1' ),
                               Item( 'expression2' ),
                               Item( 'expression3' ), 
                               Item('refresh' ) ),
                               
                 Item( 'mfn', editor = MFnChacoEditor(adapter = a), 
                       resizable = True, show_label = False ),       
                 resizable = True,
                 scrollable = True,
                 height = 0.5, width = 0.5
                    )

if __name__ == '__main__':
    fn = AnalyticalFunction()
    fn._refresh_fired()
    fn.configure_traits( view = "view_mpl", kind = 'nonmodal' )
   # fn.configure_traits( view = "view_chaco", kind = 'nonmodal' )
    


#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jun 2, 2010 by: rch

# Saved method usable for data analysis of the measured arrays
#
# - evaluation of the step histogram, plotting, ironing
#

class ExTypeSave( object ):
    
    continuity_profile_dict = Property( Dict )
    def _get_continuity_profile_dict(self):
        ''' Get the histogram and the cummulative distribution of
        the step size in all measured data arrays.
        '''
        hist_dict = {}
        for i, factor in enumerate( self.factor_list ):
            data_arr = self.data_array[:,i]
            delta_arr = fabs( data_arr[1:] - data_arr[:-1] )
            ydata, xdata = histogram( delta_arr, bins = 1000, normed = True )
            cs_hg = cumsum( ydata )
            norm_cs_hg = hstack( [zeros( (1,), dtype = 'float_' ), cs_hg / fabs( cs_hg[-1] )] )
            max_delta = fabs( xdata[-1] )
            norm_xdata = xdata / max_delta
            #print 'hy',norm_cs_hg
            hist_dict[ factor ] = ( norm_xdata, norm_cs_hg, max_delta )
        return hist_dict
       
    def _plot_continuity_profiles( self, axes ):
    
        axes.set_xlabel( 'normalized delta' )
        axes.set_ylabel( 'normalized cummulative frequency' )

        factors = list(self.continuity_profile_dict.keys())

        for factor in factors:
            hist = self.continuity_profile_dict[ factor ]
            axes.plot( hist[0], hist[1]
                       # color = c, linewidth = w, linestyle = s 
                       )
                    
        axes.legend( factors )
        
    # Factor used to cut off the delta in the measured field
    # If there is a too big step in the array it should be 
    # removed since it most probably corresponds to 
    # disturbance in the measuring process 
    #
    cutoff_delta_factor = Float( 100, 
                      auto_set = False, enter_set = True,
                      ironing_param = True )
    
    
    data_array_ironed = Property( Array( float ),
                                   depends_on = 'data_array, +ironing_param, +axis_selection' )
    @cached_property
    def _get_data_array_ironed(self):
        '''remove the jumps in the displacement curves 
        due to resetting the displacement gauges. 
        '''
        print('*** curve ironing 2 activated ***')
        
        # each column from the data array corresponds to a measured parameter 
        # e.g. displacement at a given point as function of time u = f(t))
        #
        data_array_ironed = copy( self.data_array )
        
        for idx, factor in enumerate( self.factor_list ):

            if self.names_and_units[0][ idx ] == 'Kraft' or \
                self.names_and_units[0][ idx ] == 'Bezugskanal' or \
                self.names_and_units[0][ idx ] == 'Weg':
                continue
            
            rel_delta, cum_hg, max_delta = self.continuity_profile_dict[ factor ]

            rel_delta_median = interp( 0.5, cum_hg, rel_delta )

            print('factor', factor)
            print('rel_delta_median', rel_delta_median)
            print('cutoff fraction', ( 1.0 / self.cutoff_delta_factor ))

#            if rel_delta_median <= ( 1.0 / self.cutoff_delta_factor ):
#                
            print('ironing')

            data_arr = copy( data_array_ironed[:,idx] )
            delta_median = rel_delta_median * max_delta
            print('delta_median', delta_median)
            delta_threshold = delta_median * self.cutoff_delta_factor
            print('delta_threshold', delta_threshold)

            # get the difference between each point and its successor
            delta_arr =  fabs( data_arr[1:] - data_arr[:-1] )
            print('delta_arr.shape', delta_arr.shape)
            # get the indexes in 'data_column' after which a 
            # jump exceeds the defined tolerance criteria
            delta_iron_idx = where( delta_arr > delta_threshold )[0]
            print('delta_iron_idx.shape', delta_iron_idx) 
            # glue the curve at each jump together
            for jidx in delta_iron_idx:
                # get the offsets at each jump of the curve
                shift = data_arr[jidx+1] - data_arr[jidx]
                #print 'shifting index', jidx, 'by', shift
                # shift all succeeding values by the calculated offset
                data_arr[jidx+1:] -= shift

            data_array_ironed[:,idx] = data_arr

        return data_array_ironed



'''
Created on 15. 2. 2014

@author: Vancikv
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCBar

import numpy as np

import pickle

from bmcs_beam.mxn.utils import \
    get_outfile

def test_cross_section_mn():
    '''Test the moment and normal force calculated for a cross section.
    Rectangular cross section with steel reinforcement.
    '''

    bar = RLCBar(x=0.1, z=0.05, material='bar_d10')
    bar.material_.area = 0.0002

    ge = MCSGeoRect(height=0.5, width=0.3)
    cs = CrossSection(reinf=[bar],
                         matrix_cs=MatrixCrossSection(geo=ge,
                                        n_cj=20, material='default_mixture',
                                        material_law='constant'),
                         eps_lo=0.002,
                         eps_up=-0.0033,
                         )

    cs_file = get_outfile(folder_name='.mxn',
                          file_name='test02_cs.pkl')
    assert np.allclose([cs.M, cs.N], [605.63085424909093, -4763.6924315440474])
    pickle.dump(cs, open(cs_file, 'wb'), 1)

    bar.x = 0.15
    bar.z = 0.15
    assert np.allclose([cs.M, cs.N], [595.51085561046102, -4806.0924290168105])
    cs.eps_lo = 0.010
    assert np.allclose([cs.M, cs.N], [393.29047407596528, -1821.7451022853184])

#     loaded_cs = pickle.load(open(cs_file, 'rb'))
#     assert np.allclose([loaded_cs.M, loaded_cs.N], [605.63085424909093, -4763.6924315440474])

if __name__ == '__main__':
    test_cross_section_mn()
